//----------------------------------------------------------------------------
//  Project Ralph Calculus Concepts
//  iMedia Productions Inc.
//  Copyright  1998. All Rights Reserved.
//
//  SUBSYSTEM:    Generic (Miscellaneous) Functions
//  FILE:         MiscFunc.cpp
//  AUTHOR:       Curt M. Swartzendruber, Consultant (swartzc@palmnet.net)
//  Version:		1.5.2 1/15/98 - Curt M. Swartzendruber, Original
//
//  OVERVIEW
//  ~~~~~~~~
//  Program to mix wave files
//
//----------------------------------------------------------------------------
#include "miscfunc.h"		// Header for Misc Functions (this)
#include <string.h>
#include <windows.h>
#include <math.h>
#include <values.h>
#include <stdio.h>

///////////////////////////////////////////////////////////////////////////////
// ExtractPath Function
//
//	DESCRIPTION:	Extracts the path from the passed string (i.e. steps down to
//						the next "\"
// PARAMETERS:		path - <volatile> The starting path.  String is modified
//							    by reducing the path to the next '\' or until the
//								 root directory is reached.
// RETURNS:			true if file has been found
//						false if file has NOT been found
//
//						path - returns the modified path
///////////////////////////////////////////////////////////////////////////////
bool ExtractPath(char* path)
{
	bool isPathValid = true;
   unsigned int i;
//
// Start at the end of the string down
//
  	for (i=strlen(path)-2; i>0; i--) {
     	if (path[i]==':') {
        	isPathValid = false;
         break;
      } else if (path[i]=='\\') {
        	break;
      }

      path[i] = '\0';
   }

   return isPathValid;
}


///////////////////////////////////////////////////////////////////////////////
// LocateFileWithinPath Function
//
//	DESCRIPTION:	Scans the entire path from the specified directory down to
//						the root until the file is found
// PARAMETERS:		file - The name of the file to locate
//						path - <volatile> The starting path.  String is modified
//							    until file found or until cleared.  Must not include
//								 the filename and must be terminated by a "\"
//						pathSize - Size of the path char string.
// RETURNS:			true if file has been found
//						false if file has NOT been found
//
//						path - returns the modified path where the file is located
///////////////////////////////////////////////////////////////////////////////
bool LocateFileWithinPath(const char* file, char* path, const int pathSize)
{
	HANDLE searchHandle;
   WIN32_FIND_DATA findFileData;
	bool searchFailed;
   bool rtncd;
   char *pathAndFile = new char[pathSize + strlen(file) + 1];
//
// Loop until we find the file or it fails
//
   do {
   	searchFailed = false;
   //
   // Add the file to pathAndFile - it has always been stripped out at this point
   //
   	strcpy(pathAndFile, path);
   	strcat(pathAndFile, file);
   //
   // See if the file exists at this path
   //
		searchHandle = FindFirstFile(pathAndFile, &findFileData);
	//
   // Determine what the return code would be
	//
      rtncd = (!(searchHandle==INVALID_HANDLE_VALUE));
   //
   // If exec failed drop down on subdir and try again - until we hit the root
   //
   	if (!rtncd) {

      	searchFailed = !ExtractPath(path);
      } else {
	      FindClose(searchHandle);
      }

   } while ( (!rtncd) && (!searchFailed) );

   delete[] pathAndFile;

   return rtncd;
}


///////////////////////////////////////////////////////////////////////////////
// StripTrailingZerosFromNumber Function
//
//	DESCRIPTION:	Strips all trailing zeros from the passed number string
// PARAMETERS:		numberStr - The string to be modified
// RETURNS:			Nothing
///////////////////////////////////////////////////////////////////////////////
void StripTrailingZerosFromNumber(char *numberStr)
{
//
// Check the text for trailing "0"
//
   unsigned int i;
   for (i=strlen(numberStr)-1; i>0; i--) {
      if (numberStr[i]!='0') break;
   }
   if (numberStr[i]!='.') i++;

   numberStr[i] = '\0';
}


///////////////////////////////////////////////////////////////////////////////
// FixedLengthNumber Function
//
//	DESCRIPTION:	Forces a number to a fixed number of characters.  Pads with
//						trailing zeros if number is to short and replaces number
//						text with "*" if number is to long
// PARAMETERS:		number - the number to conver
//						length - max length for number
//						numberStr - The string to place the number
// RETURNS:			true if successful
//						false if failed
///////////////////////////////////////////////////////////////////////////////
bool FixedLengthNumber(const float number, const int length, char *numberStr)
{
	char buf[100];
   int i;
   int decimalPos = -1;
//
// Convert the number
//
	sprintf(buf, "%f", number);
//
// Check for a '.' within the required length
//
	for (i=0; i<length && i<(int)strlen(buf); i++) {
   	if (buf[i]=='.') decimalPos = i;
      numberStr[i] = buf[i];
   }
//
// If no decimal found then fill with "*" and return error
//
	if (decimalPos==-1) {
   	for (i=0; i<length; i++) {
      	numberStr[i] = '*';
      }
      numberStr[i] = '\0';
      return false;
   }
//
// Pad remainder with '0's
//
	for (i=i; i<length; i++) {
   	numberStr[i] = '0';
   }
   numberStr[i] = '\0';

   return true;
}


///////////////////////////////////////////////////////////////////////////////
// CheckAnswer() Function
//
//	DESCRIPTION:	Compares two answer strings.
// PARAMETERS:		HWndCasInterface - HWND for the CAS Interface
//						guess - the number to conver
//						answerText - max length for number
//						multiline - is the text to be displayed as multiline
// RETURNS:			true if successful
//						false if failed
//						Modified "AnswerText" which includes the words "CORRECT"
///////////////////////////////////////////////////////////////////////////////
bool CompareAnswer(HWND HWndCasInterface, char *guess, const char *answerText, const bool multiline)
{
	return true;
}


///////////////////////////////////////////////////////////////////////////////
// CheckAnswer() Function
//
//	DESCRIPTION:	Compares two answer strings.
// PARAMETERS:		HWndCasInterface - HWND for the CAS Interface
//						guess - the number to conver
//						answerText - max length for number
//						multiline - is the text to be displayed as multiline
//						showCorrect - should show the CORRECT text
// RETURNS:			true if successful
//						false if failed
//						Modified "AnswerText" which includes the words "CORRECT"
///////////////////////////////////////////////////////////////////////////////
#define CORRECTTEXT_SINGLELINE "CORRECT!  "
#define CORRECTTEXT_MULTILINE  "CORRECT!\r\n"
bool CompareAnswer(HWND HWndCasInterface, char *guess, const char *answerText,
						 const bool multiline, const bool showCorrect)
{
	return CompareAnswer(HWndCasInterface, guess, answerText, "x", multiline, showCorrect);
}


bool CompareAnswer(HWND HWndCasInterface, char *guess, const char *answerText,
						 const char *variable, const bool multiline, const bool showCorrect)
{
	return true;
}


///////////////////////////////////////////////////////////////////////////////
// ConvertFloat() Function
//
//	DESCRIPTION:	Converts a floating point number to text. Stripping all
//						trailing "0" from the number.  If min or max float the
//						returned text will be "Not Available"
// PARAMETERS:		f - Number to convert
//						text - String to place converted number
//						size - length of the string
// RETURNS:			true if successful
//						false if failed
//						Modified "text" which contains the converted text
///////////////////////////////////////////////////////////////////////////////
bool ConvertFloat(const float f, char* text, const int size)
{
	if ( (f==MAXFLOAT) || (f==MINFLOAT) ) {
   	strncpy(text, "Not Available", size);
      return false;
   }

	char *buf = new char[100];
   sprintf(buf, "%f", f);

   ::StripTrailingZerosFromNumber(buf);

   strncpy(text, buf, size);

   delete[] buf;

   return true;
}




