//----------------------------------------------------------------------------
//  Project Ralph Calculus Concepts
//  iMedia Productions Inc.
//  Copyright  1998. All Rights Reserved.
//
//  SUBSYSTEM:    Generic (Miscellaneous) Functions
//  FILE:         MiscFunc.h
//  AUTHOR:       Curt M. Swartzendruber, Consultant (swartzc@palmnet.net)
//  Version:		1.5.2 1/15/98 - Curt M. Swartzendruber, Original
//
//  OVERVIEW
//  ~~~~~~~~
//  A collection of miscellaneous functions
//		ExtractPath - Reduces the string to the next "\" starting from the back
//						  of the string.  Must be terminated by a backslash "\"
//		LocateFileWithinPath - Scans the directory for the file until the root
//									  directory is reached.  Path must be terminated by
//									  an backslash "\".
//		StripTrailingZerosFromNumber - Takes the trailing zeros out of the passes
//						  string.
//		FixedLengthNumber - Forces a number to a fixed number of characters.  Pads
//						  with trailing zeros if number is to short and replaces
//						  number text with "*" if number is to long
//		CompareAnswer - Compares two answer strings for a match (three versions).
//						  Version 1 - Simply compares to strings
//						  Version 2 - Same as version 1 except will place "CORRECT" in
//										  answer when correct.  Assume functions are f(x).
//						  Version 3 - Same as version 2 except takes a parameter for
//										  the variable used in the function.
//		ConvertFloat - Converts a floating point number to text. Stripping all
//						  trailing "0" from the number.  If min or max float the
//						  returned text will be "Not Available"
//
//----------------------------------------------------------------------------
#include <owl\owlpch.h>

#if !defined(miscfunc_h)              // Sentry, use file only if it's not already included.
#define miscfunc_h

bool ExtractPath(char* path);
bool LocateFileWithinPath(const char* file, char* path, const int pathSize);
void StripTrailingZerosFromNumber(char* numberStr);
bool FixedLengthNumber(const float number, const int length, char *numberStr);
bool CompareAnswer(HWND HWndCasInterface, char *guess, const char *answerText,
						 const bool multiline);
bool CompareAnswer(HWND HWndCasInterface, char *guess, const char *answerText,
						 const bool multiline, const bool showCorrect);
bool CompareAnswer(HWND HWndCasInterface, char *guess, const char *answerText,
						 const char* variable, const bool multiline, const bool showCorrect);
bool ConvertFloat(const float f, char* text, const int size);

#endif
