//----------------------------------------------------------------------------
//  Project Ralph Calculus Concepts
//  iMedia Productions, Inc.
//  Copyright  1997. All Rights Reserved.
//
//  SUBSYSTEM:    Generic Application
//  FILE:         serror.cpp
//  AUTHOR:       Curt M. Swartzendruber, Consultant (swartzc@palmnet.net)
//	 HISTORY:		1.5.2 CMS, Original
//						1.5.3 10/31/97 CMS - Added DisplayResourceMessage function
//
//  OVERVIEW
//  ~~~~~~~~
//  Source file for implementation of SError (TWindow).
//
//----------------------------------------------------------------------------
#include <owl/owlpch.h>
#include <errno.h>

#include "serror.h"


//{{SError Implementation}}


SError::SError()
{
   SErrorCode = 0;
}


SError::~SError()
{
}


void SError::SetErrorCode(const int code)
{
   SErrorCode = code;
}


void SError::DisplayMessage(const int code)
{
	char buf[1024];

   SetErrorCode( code );

   switch (code) {
   	case ErrorInvalidCommandFile:
      			strcpy(buf, "Invalid Command File Structure");
               break;

      case ErrorInvalidFileSeek:
      			strcpy(buf, "LSeek failed during command file navigation");
               break;

      case ErrorCommandFileOpen:
      			strcpy(buf, "Unable to open the specified command file");
               break;

      case ErrorFileDoesNotExist:
      			strcpy(buf, "Unable to open the specified command file");
               break;

   	case ErrorInvalidParameter:
      			strcpy(buf, "Invalid parameter detected in command file");
               break;

      case ErrorInvalidParamColor:
      			strcpy(buf, "Invalid Color parameter detected in command");
               break;

   	case ErrorParamMissingQuote:
      			strcpy(buf, "Parameter missing a quote in command file");
               break;

      case ErrorInvalidLineNumber:
      			strcpy(buf, "Invalid Line number detected in command file");
               break;

      case ErrorInvalidParamBool:
      			strcpy(buf, "Invalid Boolean parameter detected in command");
               break;

      case ErrorNewScreenPlacement:
      			strcpy(buf, "The NewScreen() command is not the first command in the new block");
               break;

      case ErrorInvalidSlideCode:
      			strcpy(buf, "Invalid slide code detected in NewScreen() command");
               break;

      case ErrorResourceStringLoadFailure:
      			strcpy(buf, "Failed to load resource string");
               break;

      case ErrorEnd:
      			strcpy(buf, "Failed to locate \"End\" command");
               break;

      default:
      			wsprintf(buf, "Unrecognized error code %d", code);
               break;
   }

   MessageBox(0, buf, "Error", MB_OK | MB_ICONERROR | MB_SYSTEMMODAL);
}


void SError::DisplayMessage(const int code, const char* text)
{
   SetErrorCode( code );
//
// Display the error message
//
   MessageBox(0, text, "Error", MB_OK | MB_ICONERROR | MB_SYSTEMMODAL);
}


void SError::DisplayResourceMessage(const TModule *module, const int id)
{
	char buf[1024];
//
// Make sure module exists
//
	if (!module) {
   	MessageBox(0, "Module does not exist in call to SError::DisplayResourceMessage", "Error", MB_OK | MB_ICONERROR | MB_SYSTEMMODAL);
      return;
   }
//
// Load the string
//
   if (0==module->LoadString(id, buf, sizeof(buf))) {
   	DisplayMessage(ErrorResourceStringLoadFailure);
      return;
   }
//
// Display the error
//
	MessageBox(0, buf, "Error", MB_OK | MB_ICONERROR | MB_SYSTEMMODAL);
}


int SError::Terminate(const int code)
{
//
// Display the error message
//
   DisplayMessage(code);
//
// Tell the program to shut itself down
//
///	PostThreadMessage(GetCurrentThreadId(), WM_TERMINATE, 0, 0);
///   PostThreadMessage(GetCurrentThreadId(), CM_EXIT, 0, 0);
   ExitProcess( 0 );
//
// return the passed error
//
   return code;
}


int SError::Terminate(const int code, const char* message)
{
//
// Display the error message
//
   DisplayMessage(code, message);
//
// Tell the program to shut itself down
//
///	PostThreadMessage(GetCurrentThreadId(), WM_TERMINATE, 0, 0);
///   PostThreadMessage(GetCurrentThreadId(), CM_EXIT, 0, 0);
   ExitProcess( 0 );
//
// return the passed error
//
   return code;
}


int SError::TerminateWithErrno(void)
{
//
// Display the appropriate error message depending on errno
//
   switch (errno) {
      case EACCES :
         DisplayMessage(errno, "Permission denied");
         break;
      case EMFILE:
         DisplayMessage(errno, "Too many open files");
         break;
      case ENOENT:
         DisplayMessage(errno, "Path or file name not found");
         break;
      case ENOEXEC:
         DisplayMessage(errno, "Exec format error");
         break;
      case ENOMEM:
         DisplayMessage(errno, "Not enough memory");
         break;
   }
//
// Tell the program to shut itself down
//
///	PostThreadMessage(GetCurrentThreadId(), WM_TERMINATE, 0, 0);
///   PostThreadMessage(GetCurrentThreadId(), CM_EXIT, 0, 0);
   ExitProcess( 0 );
//
// return the passed error
//
   return errno;
}


void SError::Test(const int code)
{
   if (code) {
      Terminate(code);
   }
}


void SError::Test(const bool success, const char* text)
{
   if (!success) {
      Terminate(EFAULT, text);		// Return unknown error
   }
}

