//----------------------------------------------------------------------------
//  Project Calculus Concepts Module
//  iMedia Productions, Inc.
//  Copyright  1997. All Rights Reserved.
//
//  SUBSYSTEM:    Generic Application
//  FILE:         spalette.cpp
//  AUTHOR:       Curt M. Swartzendruber, Consultant (swartzc@palmnet.net)
//	 HISTORY:		1.5.2 Original
//	 					1.5.3	- CMS, Modified LoadBitmapFile
//
//  OVERVIEW
//  ~~~~~~~~
//----------------------------------------------------------------------------
#include <dir.h>
#include <stdio.h>
#include <string.h>
#include <fstream.h>
#include <owl/gdiobjec.h>
#include "spalette.h"
#include "serror.h"

extern char ExePath[];
extern SError *AppError;

//{{SPalette Implementation}}

//
// Palette Constructor
//
SPalette::SPalette(void)
{
   TScreenDC screenDC;
   if (screenDC.GetDeviceCaps(RASTERCAPS) & RC_PALETTE) {
      ScreenPalettized = true;
#if defined(BI_PLOT_WIN32)
      DefPalette = new THalftonePalette;
#else
      DefPalette = new TPalette( (HPALETTE)::GetStockObject(DEFAULT_PALETTE));
#endif
   } else {
      ScreenPalettized = false;
      DefPalette = 0;
   }

   Palette = 0;
   FirstFileLoaded = false;
}



//
// SPalette destructor
//
SPalette::~SPalette()
{
   if (Palette!=NULL) {
   	delete Palette;
      Palette = NULL;
   }

   if (DefPalette!=NULL) {
   	delete DefPalette;
      DefPalette = NULL;
   }
}


bool SPalette::GetScreenPalettized(void)
{
   return ScreenPalettized;
}


TPalette *SPalette::GetPalette(void)
{
   return Palette;
}


TBitmap *SPalette::LoadBitmapFile(const char far *filename)
{
   char tmpfile[256];
   TBitmap *bitmap = NULL;
//
// Load file into a DIB
//
   TDib *newDib;
   newDib = NULL;

   try {
      strcpy(tmpfile, ExePath);
		strcat(tmpfile, filename);
		newDib = new TDib( tmpfile );
	} catch(...) { ///TXGdi) {
      char buf[MAXPATH];
      sprintf(buf, "Can not load bitmap file [%s]", tmpfile);
      AppError->Terminate(-1, buf);
   }
//
// Setup the palette if desired if it has not yet been created
//
   if (ScreenPalettized) {
   //
   // Does the palette need to be created from the bitmap
   //
      if (!Palette) {
         try {
	   		Palette = new TPalette(*newDib);
            TScreenDC().SelectObject(*Palette);
            TScreenDC().RealizePalette();
         } catch (...) {
            Palette = DefPalette;
         }
      }
   }
//
// Create and return the address of the TBitmap
//
   if (newDib) {
   	if (ScreenPalettized) {
	      bitmap = new TBitmap(*newDib, Palette);
   	} else {
      	bitmap = new TBitmap(*newDib);
	   }

	   delete newDib;
   }

   return bitmap;
}



void SPalette::PaletteChanged(TWindow *wnd, HWND hWndPalChg)
{
	HWND hWnd= wnd->GetHandle();

   if ( (hWnd) && (hWndPalChg!=hWnd)) {	  	// Make sure it is not this instance
   	if (ScreenPalettized && Palette) {		// Make sure we are dealing with palettes
   		TClientDC clientDC(*wnd);
#if defined(BI_PLAT_WIN16)
	    	Palette->UnrealizeObject();
#endif
			clientDC.SelectObject(*Palette, true);
	   	bool needsUpdate = clientDC.RealizePalette() > 0;
	   	if (needsUpdate) {
		      wnd->Invalidate(false);
      	}
		}
   }
}


bool SPalette::QueryNewPalette(TWindow *wnd)
{
   bool rtncd = false;
//
// Update to the current palette
//
   if (ScreenPalettized && Palette) {
      TClientDC dc(*wnd);
#if defined(BI_PLAT_WIN16)
      Palette->UnrealizeObject();
#endif
      dc.SelectObject(*Palette);
      bool needsUpdate = dc.RealizePalette()>0;
      if (needsUpdate) {
         wnd->Invalidate(false);
         rtncd = true;
      }
   }

   return rtncd;
}


void SPalette::SelectPalette(TDC &dc)
{
//
// SelectObject(Palette) only if there is a palette
//
   if (Palette) {
      dc.SelectObject(*Palette);
      dc.RealizePalette();
   }
}
