//----------------------------------------------------------------------------
//  Project Ralph Calculus Concepts
//  iMedia Productions, Inc.
//  Copyright  1998. All Rights Reserved.
//
//  SUBSYSTEM:    Generic Application
//  FILE:         SRadioBn.cpp
//  AUTHOR:       Curt M. Swartzendruber, Consultant (swartzc@palmnet.net)
//  HISTORY:		1.5.2 (04/22/98) Original
//
//  OVERVIEW
//  ~~~~~~~~
//  Class definition for SRadioButton (TRadioButton).
//
//----------------------------------------------------------------------------
#include <owl/owlpch.h>
#include <owl/groupbox.h>
#include <checks.h>

#pragma hdrstop

#include "rdodawnd.h"
#include "SRadioBn.h"

#include "spalette.h"
extern SPalette *AppPalette;
extern char ExePath[];

//
// Build a response table for all messages/commands handled by the application.
//
DEFINE_RESPONSE_TABLE1(SRadioButton, TRadioButton)
//{{SRadioButtonRSP_TBL_BEGIN}}
  //EV_WM_LBUTTONDOWN,
  //EV_WM_ERASEBKGND,
//{{SRadioButtonRSP_TBL_END}}
END_RESPONSE_TABLE;


//{{SRadioButton Implementation}}


SRadioButton::SRadioButton (TWindow* parent, int id, const char* bmpFilename,
   		   			 int x, int y, int w, int h, const char* waveFilename,
	      	          TGroupBox* group, TModule* module)
: TRadioButton(parent, id, bmpFilename, x, y, w, h, group, module)
{
   if (waveFilename==NULL) {
      WavSound = NULL;
   } else {
   	WavSound = new SSound(Parent, waveFilename, GetModule());
   }
//
// Pre-load the UP and DOWN bitmaps
//
   BmImage = AppPalette->LoadBitmapFile(bmpFilename);
	ImageDC = new TMemoryDC;
   ImageDC->SelectObject(*BmImage);
//
// Store the width and height
//
   ImageWidth = BmImage->Width();
	ImageFrameHeight = (BmImage->Height()-1)/2;

	CheckedTopLeft = new TPoint(0,0);							// Checked state topleft
	UnCheckedTopLeft = new TPoint(0,ImageFrameHeight+1);	// UnChecked state topleft

   ButtonWidth = ImageWidth;
   ButtonHeight = ImageFrameHeight;
//
// Tell system that the push-button is owner-drawn only if bitmaps were loaded sucessfully
//
	if (BmImage) {
		Attr.Style |= BS_OWNERDRAW;  	/// PROBLEM - Comment line out and it works
	}

   Attr.Style |= WS_TABSTOP;

   Checked = false;
}


SRadioButton::~SRadioButton ()
{
	 Destroy();
//
// Remove the allocated bitmaps (only if they were loaded)
//
	delete UnCheckedTopLeft;
   delete CheckedTopLeft;

	delete BmImage;
   delete ImageDC;
}


void SRadioButton::ODADrawEntire(DRAWITEMSTRUCT far& drawInfo)
{
   TRACE( "SRadioButton::ODADrawEntire " << drawInfo.CtlID << " " << drawInfo.itemState );
	TRadioButton::ODADrawEntire(drawInfo);

	DrawMyButton(drawInfo);
}


void SRadioButton::ODAFocus(DRAWITEMSTRUCT far& drawInfo)
{
   TRACE( "SRadioButton::ODAFocus " << drawInfo.CtlID << " " << drawInfo.itemState );
	TRadioButton::ODAFocus(drawInfo);

	DrawMyButton(drawInfo);
}


void SRadioButton::ODASelect(DRAWITEMSTRUCT far& drawInfo)
{
   TRACE( "SRadioButton::ODASelect " << drawInfo.CtlID << " " << drawInfo.itemState);
	TRadioButton::ODASelect(drawInfo);
   TYPESAFE_DOWNCAST(Parent,TRadioODAWindow)->OldSelection = TYPESAFE_DOWNCAST(Parent,TRadioODAWindow)->Selection;
   TYPESAFE_DOWNCAST(Parent,TRadioODAWindow)->Selection = drawInfo.CtlID;

	DrawMyButton(drawInfo);
   Parent->ChildWithId(TYPESAFE_DOWNCAST(Parent,TRadioODAWindow)->OldSelection)->Invalidate();
}


int SRadioButton::BitmapWidth(void)
{
   return BmImage->Width();
}


int SRadioButton::Width(void)
{
   return ButtonWidth;
}


int SRadioButton::BitmapHeight(void)
{
   return BmImage->Height();
}


int SRadioButton::Height(void)
{
   return ButtonHeight;
}


void SRadioButton::DrawMyButton(DRAWITEMSTRUCT far& drawInfo)
{
	TClientDC dc(*this);

   AppPalette->SelectPalette(dc);

   DrawInfo = drawInfo;
//
// Set some common values
//
   ButtonWidth = drawInfo.rcItem.right-drawInfo.rcItem.left;
   ButtonHeight= drawInfo.rcItem.bottom-drawInfo.rcItem.top;

   dc.SetStretchBltMode( COLORONCOLOR );
//
// Determine which mode to draw, See if it is SELECTED
//
	if ((TYPESAFE_DOWNCAST(Parent,TRadioODAWindow))->Selection==drawInfo.CtlID) {
         dc.StretchBlt(drawInfo.rcItem.left, drawInfo.rcItem.top,
                       ButtonWidth, ButtonHeight, *ImageDC, CheckedTopLeft->x,
                       CheckedTopLeft->y, ImageWidth, ImageFrameHeight, SRCCOPY);

         Checked = true;
	} else {
	//
	// Draw the Unchecked state
	//
		dc.StretchBlt(drawInfo.rcItem.left, drawInfo.rcItem.top,
					     ButtonWidth, ButtonHeight, *ImageDC, UnCheckedTopLeft->x,
                    UnCheckedTopLeft->y, ImageWidth, ImageFrameHeight, SRCCOPY);

      Checked = false;
   }
//
// Draw the focus rect
//
   if (drawInfo.itemState & ODS_FOCUS) {
		dc.DrawFocusRect(drawInfo.rcItem);

   }
}


void SRadioButton::EvLButtonDown(uint modKeys, TPoint& point)
{
   TRACE("SRadioButton::EvLButtonDown");
   TRadioButton::EvLButtonDown(modKeys, point);
//
// Make the sound
//
   if (WavSound) WavSound->Play(SND_FROMSTART|SND_ASYNC|SND_NOSTOP);
}

/*
void SRadioButton::SetupWindow()
{
	TRadioButton::SetupWindow();
//
// Change the cursor for this window/control
//
   SetCursor(GetModule(), TResId(IDC_MOUSELEFTBUTTON));
}


bool SRadioButton::Create()
{
	bool rtncd = TRadioButton::Create();

   if (WavSound) {
   	WavSound->Create();
   }

   return rtncd;
}

*/
void SRadioButton::Paint(TDC& dc, bool erase, TRect& rect)
{
   TRACE( "SRadioButton::Paint" );
	TRadioButton::Paint(dc, erase, rect);

   if (rect.Touches( TRect(Attr.X, Attr.Y, Attr.X+Attr.W, Attr.Y+Attr.H) )) {
		dc.SetStretchBltMode( COLORONCOLOR );

	   if (GetCheck()==BF_CHECKED) {
			dc.StretchBlt(Attr.X, Attr.Y, Attr.W, Attr.H, *ImageDC,
				           CheckedTopLeft->x, CheckedTopLeft->y, ImageWidth,
      		  	        ImageFrameHeight, SRCCOPY);
   	} else {
			dc.StretchBlt(Attr.X, Attr.Y, Attr.W, Attr.H, *ImageDC,
				           UnCheckedTopLeft->x, UnCheckedTopLeft->y, ImageWidth,
   	   	  	        ImageFrameHeight, SRCCOPY);
	   }
   }
}


void SRadioButton::PaintALL(TDC& dc, bool /*erase*/, TRect& /*rect*/)
{
   TRACE( "SRadioButton::PaintALL" );
	dc.SetStretchBltMode( COLORONCOLOR );

   if (GetCheck()==BF_CHECKED) {
		dc.StretchBlt(Attr.X, Attr.Y, Attr.W, Attr.H, *ImageDC,
			           CheckedTopLeft->x, CheckedTopLeft->y, ImageWidth,
      	  	        ImageFrameHeight, SRCCOPY);
   } else {
		dc.StretchBlt(Attr.X, Attr.Y, Attr.W, Attr.H, *ImageDC,
			           UnCheckedTopLeft->x, UnCheckedTopLeft->y, ImageWidth,
      	  	        ImageFrameHeight, SRCCOPY);
   }
}


bool SRadioButton::EvEraseBkgnd(HDC)
{
   TRACE("SRadioButton::EvEraseBkgnd");
	return true;		// Do not erase the background
}


bool SRadioButton::Touches( TPoint pnt )
{
   TRect rect(Attr.X, Attr.Y, Attr.X+Attr.W, Attr.Y+Attr.H);

	return rect.Contains(pnt);
}

UINT SRadioButton::GetCheck() const
{
	if (Checked)
   	return BF_CHECKED;
   else
   	return BF_UNCHECKED;
}
